@@ -52,7 +52,7 @@ void insertgroup(const char * name, int first, int last, const char * desc,
     a = &active;
     while(a) {
 	if (*a) {
-	    c = strcmp((*a)->name, name);
+	    c = strcasecmp((*a)->name, name);
 	    if (c<0)
 		a = &((*a)->left);
 	    else if (c>0)
@@ -97,11 +97,26 @@ void insertgroup(const char * name, int first, int last, const char * desc,
 /*
  * check whether group is represented in interesting.groups
  */
-int isinteresting( char * groupname ) {
-    struct stat st;
+int isinteresting( const char * groupname ) {
+    DIR * d;
+    struct dirent * de;
 
-    sprintf( s, "%s/interesting.groups/%s", spooldir, groupname );
-    return(stat( s, &st ) + 1);
+    sprintf( s, "%s/interesting.groups", spooldir );
+    d = opendir( s );
+    if ( !d ) {
+	syslog( LOG_ERR, "Unable to open directory %s: %m", s );
+	printf( "Unable to open directory %s\n", s );
+	return FALSE;
+    }
+
+    while( ( de = readdir( d ) ) != NULL ) {
+	if ( strcasecmp( de->d_name, groupname ) == 0 ) {
+	    closedir( d );
+	    return TRUE;
+	}
+    }
+    closedir( d );
+    return FALSE;
 }
 
 /* find a group by name */
@@ -111,11 +126,13 @@ struct newsgroup * findgroup(const char* name) {
 
     a = active;
     while ( a ) {
-	c = strcmp( a->name, name );
-	if ( c < 0 )
+	c = strcasecmp( a->name, name );
+	if ( c < 0 ) {
 	    a = a->left;
-	else if ( c > 0 )
+	}
+	else if ( c > 0 ) {
 	    a = a->right;
+	}
 	else
 	    return a;
     }
@@ -127,6 +144,10 @@ static void freeactive( struct newsgroup * g ) {
     if ( g ) {
 	freeactive( g->right );
 	freeactive( g->left );
+	if ( g->desc )
+	    free( g->desc );
+	if ( g->name )
+	    free( g->name );
 	free( (char*)g );
     }
 }
@@ -140,7 +161,7 @@ static void freeactive( struct newsgroup * g ) {
 
    1 2 3 4 5 6 7
 
-         1
+	 1
      1   0   1
    1 0 1 0 1 0 1
    | | | | | | + no zeroes: leaf node
@@ -155,7 +176,7 @@ static void freeactive( struct newsgroup * g ) {
    care, a _perfectly_ balanced tree */
 
 void readactive( void ) {
-    char * p, *q, *r, *lastline, *desc;
+    char * p, *q, *r, *lastline, *desc = 0;
     int first, last;
     time_t age;
     FILE * f;
@@ -177,7 +198,7 @@ void readactive( void ) {
 	syslog( LOG_ERR, "can't stat %s: %m", s );
 	return;
     } else if ( !S_ISREG( st.st_mode ) ) {
-	syslog( LOG_ERR, "%s not a regular file" );
+	syslog( LOG_ERR, "%s not a regular file", s );
 	return;
     }
     stuff = critmalloc( st.st_size+2, "Reading group info" );
@@ -186,7 +207,7 @@ void readactive( void ) {
 	if ( n < st.st_size )
 	    syslog( LOG_ERR,
 		    "Groupinfo file truncated while reading: %d < %d .",
-		    n, st.st_size );
+		    n, (int) st.st_size );
 	fclose( f );
     }
     else {
@@ -201,7 +222,7 @@ void readactive( void ) {
 
     /* delete spurious 0-bytes */
     while (( p = memchr( stuff, '\0', st.st_size ) ) != NULL )
-        *p = ' ';       /* \n might be better, but produces more errors */
+	*p = ' ';       /* \n might be better, but produces more errors */
 
     line = 1;
     for( l=0; l<32; levels[l++] = NULL )
@@ -212,46 +233,46 @@ void readactive( void ) {
     while ( (p < (stuff+n)) && (( q = strchr( p, '\n' )) != NULL ) ) {
 	*q = '\0' ;
 	lastline = p;
-        if (( ( r = strchr( p, ' ' )) != NULL ) && isalnum((int)*p) ) {
-            *r = '\0' ;
-            p = r + 1 ;
-        } else
-            error = 1;
-        if ( !error ) {
-            last = *p ? strtol( p, &r, 10 ) : 1;
-            if ( r == p )
-                error = 1;
-            else {
-                p = r;
-                first = *p ? strtol( p, &r, 10 ) : 1;
-                if ( r == p )
-                    error = 1;
-                else {
-                    p = r;
-                    age = *p ? strtol( p, &r, 10 ) : 0;
-                    if ( r == p )
-                        error = 1;
+	if (( ( r = strchr( p, ' ' )) != NULL ) && isalnum((unsigned char)*p)) {
+	    *r = '\0' ;
+	    p = r + 1 ;
+	} else
+	    error = 1;
+	if ( !error ) {
+	    last = *p ? strtol( p, &r, 10 ) : 1;
+	    if ( r == p )
+		error = 1;
+	    else {
+		p = r;
+		first = *p ? strtol( p, &r, 10 ) : 1;
+		if ( r == p )
+		    error = 1;
+		else {
+		    p = r;
+		    age = *p ? strtol( p, &r, 10 ) : 0;
+		    if ( r == p )
+			error = 1;
 		    else
 			p = r;
-                }
-            }
-        }
+		}
+	    }
+	}
 
-        if ( !error ) {
-            while (p && *p && isspace((int)*p))
-                p++;
-            if ( p && *p )
-                desc = strdup( p );
-            else
-                error = 1;
-        }
+	if ( !error ) {
+	    while (p && *p && isspace((unsigned char)*p))
+		p++;
+	    if ( p && *p )
+		desc = strdup( p );
+	    else
+		error = 1;
+	}
 
-        if ( error ) {
-            syslog( LOG_ERR,
+	if ( error ) {
+	    syslog( LOG_ERR,
 		    "Groupinfo file possibly truncated or damaged: %s\n",
 		    lastline);
-            error = 0;
-        } else {
+	    error = 0;
+	} else {
 	    g = (struct newsgroup *) critmalloc(sizeof(struct newsgroup),
 						"reading groupinfo" );
 	    g->right = g->left = NULL;
@@ -261,25 +282,25 @@ void readactive( void ) {
 	    g->name = strdup(lastline);
 	    g->desc = desc;
 
-            /* Inserting newsgroup into tree */
-            for( l=0; (line&(1<<l))==0; l++ )
-                ;
-            if ( l ) {
-                g->right = levels[l-1];
-            }
-            if ( active == g->right )
-                active = g;
-            if ( l<31 && levels[l+1] && !levels[l+1]->left )
-                levels[l+1]->left = g;
-            levels[l] = g;
+	    /* Inserting newsgroup into tree */
+	    for( l=0; (line&(1<<l))==0; l++ )
+		;
+	    if ( l ) {
+		g->right = levels[l-1];
+	    }
+	    if ( active == g->right )
+		active = g;
+	    if ( l<31 && levels[l+1] && !levels[l+1]->left )
+		levels[l+1]->left = g;
+	    levels[l] = g;
 
-            p = strchr( p, '\n' );
-            if ( p )
-                *p++ = '\0';
-            line++;
+	    p = strchr( p, '\n' );
+	    if ( p )
+		*p++ = '\0';
+	    line++;
 	}
 
-        p = q + 1;
+	p = q + 1;
     }
     free( stuff );
 
@@ -306,27 +327,21 @@ void readactive( void ) {
 /*
  * returns 1 if something could be written, 0 otherwise
  */
-static int helpwriteactive ( struct newsgroup *g, FILE * f ) {
+static void helpwriteactive ( struct newsgroup *g, FILE * f ) {
     if (g) {
-        if ( helpwriteactive (g->right, f) ) {
-            if ( fprintf(f, "%s %d %d %lu %s\n", g->name, g->last, g->first,
-                 g->age, g->desc && *(g->desc) ? g->desc : "-x-" ) ) {
-        	if ( helpwriteactive (g->left, f) )
-		    return 1;
-		else
-		    return 0;
-	    } else
-		return 0;
-	} else
-	    return 0;
+	helpwriteactive(g->right, f);
+	fprintf(f, "%s %lu %lu %lu %s\n", g->name, g->last, g->first,
+		(unsigned long)g->age,
+		g->desc && *(g->desc) ? g->desc : "-x-" );
+	helpwriteactive (g->left, f);
     }
-    return 1;
 }
 
-void writeactive( void ) {
+void writeactive( int forcedwrite ) {
     FILE * a;
     char c[PATH_MAX];
-    int err;
+    int newlen, oldlen;
+    struct stat st;
 
     strcpy(s, spooldir);
     strcat(s, "/leaf.node/groupinfo.new");
@@ -335,16 +350,25 @@ void writeactive( void ) {
 	syslog( LOG_ERR, "cannot open new groupinfo file: %m" );
 	return;
     }
-    err = helpwriteactive( active, a );
+    helpwriteactive( active, a );
     fclose( a );
+    if ( stat( s, &st ) == 0 )
+	newlen = st.st_size ;
+    else
+	newlen = 0;
 
-    if ( err != 0 ) {
-	strcpy(c, spooldir);
-        strcat(c, "/leaf.node/groupinfo");
+    strcpy(c, spooldir);
+    strcat(c, "/leaf.node/groupinfo");
+    if ( stat( c, &st ) == 0 )
+	oldlen = st.st_size ;
+    else
+	oldlen = 0;
+
+    if ( ( newlen > oldlen-10000 ) || forcedwrite ) {
 	rename( s, c );
     } else {
 	syslog( LOG_ERR, "error writing groupinfo file (disk full?)" );
-	unlink(s);
+	unlink( s );
     }
 }
 
@@ -369,7 +393,8 @@ void fakeactive( void ) {
 	return;
     }
     while ( (de = readdir(d)) ) {
-	if ( isalnum((int)*(de->d_name) ) && chdirgroup(de->d_name, FALSE) ) {
+	if ( isalnum((unsigned char)*(de->d_name) ) &&
+	     chdirgroup(de->d_name, FALSE) ) {
 	    /* get first and last article from the directory. This is
 	     * the most secure way to get to that information since the
 	     * .overview files may be corrupt as well
@@ -380,7 +405,7 @@ void fakeactive( void ) {
 
 	    ng = opendir( "." );
 	    while ( ( nga = readdir( ng ) ) != NULL ) {
-		if ( isdigit ((int) *(nga->d_name ) ) ) {
+		if ( isdigit ((unsigned char) *(nga->d_name ) ) ) {
 		    i = strtol( nga->d_name, NULL, 10 );
 		    if ( i < first )
 			first = i;
@@ -399,4 +424,5 @@ void fakeactive( void ) {
 	    insertgroup( de->d_name, first, last, NULL, 0 );
 	}
     }
+    closedir( d );
 }
