@@ -73,9 +73,9 @@ int parse_line ( char *l, char * param, char * value ) {
     else
 	*q++ = '\0';
     /* skipping leading blanks or tabs */
-    while ( isspace((int)*p) )
+    while ( isspace((unsigned char)*p) )
 	p++;
-    while ( isspace((int)*q) )
+    while ( isspace((unsigned char)*q) )
 	q++;
     strncpy( param, p, TOKENSIZE );
     strncpy( value, q, TOKENSIZE );
@@ -85,11 +85,11 @@ int parse_line ( char *l, char * param, char * value ) {
     /* skipping trailing blanks or tabs */
     p = param + strlen(param) - 1;
     q = value + strlen(value) - 1;
-    while ( isspace((int)*p) && ( p > param ) ) {
+    while ( isspace((unsigned char)*p) && ( p > param ) ) {
 	*p = '\0';
 	p--;
     }
-    while ( isspace((int)*q) && ( q > value ) ) {
+    while ( isspace((unsigned char)*q) && ( q > value ) ) {
 	*q = '\0';
 	q--;
     }
@@ -100,10 +100,12 @@ int parse_line ( char *l, char * param, char * value ) {
 05/25/97 - T. Sweeney - Modified to read user name and password for AUTHINFO.
                         Security questionable as password is stored in
                         plaintext in insecure file.
+1999-07-15 - Matthias Andree
+             Set p and q defaults to 0 
 */
 
 int readconfig( void ) {
-    struct serverlist *p, *q ;
+    struct serverlist *p = 0, *q = 0;
     struct rlimit corelimit ;
     struct expire_entry *ent = NULL, *prev = NULL;
     FILE * f;
@@ -128,8 +130,8 @@ int readconfig( void ) {
 	        if ( p ) {
 		    p->username = strdup( value );
 		    if ( debugmode )
-	        	syslog( LOG_DEBUG, "config: username is %s",
-				p->username );
+	        	syslog( LOG_DEBUG, "config: found username for %s",
+				p->name );
 		}
 		else
 		    syslog( LOG_ERR, "config: no server for username %s",
@@ -139,15 +141,15 @@ int readconfig( void ) {
 		if ( p ) {
 		    p->password = strdup( value );
 		    if ( debugmode )
-			syslog( LOG_DEBUG, "config: password is %s",
-				p->password);
+			syslog( LOG_DEBUG, "config: found password for %s",
+				p->name );
 		}
 		else
 		    syslog( LOG_ERR, "config: no server for password" );
 	    }
 	    else if ( strcmp( "timeout", param ) == 0 ) {
 		if ( p ) {
-		    p->timeout = atoi( value );
+		    p->timeout = strtol( value, NULL, 10 );
 		    if ( debugmode )
 			syslog( LOG_DEBUG, "config: timeout is %d seconds",
 				p->timeout );
@@ -157,8 +159,8 @@ int readconfig( void ) {
 	    }
 	    else if ( strcmp ( "create_all_links", param ) == 0 ) {
 		if ( value && strlen( value ) ) {
-		    create_all_links = 1;
-		    if ( debugmode )
+		    create_all_links = strtol( value, NULL, 10 );
+		    if ( create_all_links && debugmode )
 			syslog( LOG_DEBUG,
 				"config: link articles in all groups" );
 		}
@@ -173,7 +175,8 @@ int readconfig( void ) {
 		    syslog( LOG_DEBUG, "config: filterfile is %s", value );
 		filterfile = strdup( value );
 	    }
-	    else if ( strcmp ( "hostname", param ) == 0 ) {
+	    else if ( ( strcmp ( "hostname", param ) == 0 ) ||
+		      ( strcmp ( "fqdn", param ) == 0 ) ) {
 		if ( debugmode )
 		    syslog( LOG_DEBUG, "config: hostname is %s", value );
 		owndn = strdup( value );
@@ -181,54 +184,54 @@ int readconfig( void ) {
 	    else if ( ( strcmp ( "maxcrosspost", param ) == 0 ) ||
 		      ( strcmp ( "maxgroups", param ) == 0 ) ) {
 		/* maxgroups is for compatibility with leafnode+ */
-		crosspostlimit = atoi( value );
+		crosspostlimit = strtol( value, NULL, 10 );
 		if ( debugmode )
 		    syslog( LOG_DEBUG, "config: crosspostlimit is %d groups",
 			    crosspostlimit );
 	    }
 	    else if ( strcmp( "maxlines", param ) == 0 ) {
-		maxlines = atoi( value );
+		maxlines = strtol( value, NULL, 10 );
 		if ( debugmode )
 		    syslog( LOG_DEBUG, "config: postings have max. %d lines",
 			    maxlines );
 	    }
             else if ( strcmp( "minlines", param ) == 0 ) {
-                minlines = atoi( value );
+                minlines = strtol( value, NULL, 10 );
                 if ( debugmode )
                     syslog( LOG_DEBUG, "config: postings have min. %d lines",
                             minlines );
             }
-            else if ( ( strcmp( "maxbytes", param ) == 0 ) && !maxlines ) {
-                maxbytes = atoi( value );
+            else if ( strcmp( "maxbytes", param ) == 0 ) {
+                maxbytes = strtol( value, NULL, 10 );
                 if ( debugmode )
                     syslog( LOG_DEBUG,
 			    "config: postings have max. %d bytes", maxbytes);
             }
 	    else if ( strcmp( "debugmode", param ) == 0 ) {
-		debugmode = atoi( value );
+		debugmode = strtol( value, NULL, 10 );
 		if ( debugmode )
 		    syslog( LOG_DEBUG, "config: debugmode is %d", debugmode );
 	    }
 	    else if ( strcmp( "delaybody", param ) == 0 ) {
-		delaybody = atoi( value );
+		delaybody = strtol( value, NULL, 10 );
 		if ( debugmode )
 		    syslog( LOG_DEBUG, "config: delaybody is %d (default 0)",
 			    delaybody );
 	    }
 	    else if ( strcmp( "timeout_short", param ) == 0 ) {
-		timeout_short = atoi( value );
+		timeout_short = strtol( value, NULL, 10 );
 		if ( debugmode )
 		    syslog( LOG_DEBUG, "config: timeout_short is %d days",
 			    timeout_short );
 	    }
 	    else if ( strcmp( "timeout_long", param ) == 0 ) {
-		timeout_long = atoi( value );
+		timeout_long = strtol( value, NULL, 10 );
 		if ( debugmode )
 		    syslog( LOG_DEBUG, "config: timeout_long is %d days",
 			    timeout_long );
 	    }
 	    else if ( strcmp( "timeout_active", param ) == 0 ) {
-		timeout_active = atoi( value );
+		timeout_active = strtol( value, NULL, 10 );
 		if ( debugmode )
 		    syslog( LOG_DEBUG, "config: timeout_active is %d days",
 			    timeout_active );
@@ -236,9 +239,9 @@ int readconfig( void ) {
 	    else if ( strncmp( "groupexpire", param, 11 ) == 0 ) {
 		char * m;
 		m = param;
-		while ( !(isspace((int)*m)) )
+		while ( !(isspace((unsigned char)*m)) )
 		    m++;
-		while ( isspace((int)*m) )
+		while ( isspace((unsigned char)*m) )
 		    m++;
 		if ( m && *m ) {
 		    i = time(NULL)-(time_t)( SECONDS_PER_DAY *atol(value));
@@ -257,18 +260,18 @@ int readconfig( void ) {
 	    else if ( ( strcmp( "maxage", param ) == 0 ) ||
 		      ( strcmp( "maxold", param ) == 0 ) ) {
 		/* maxold is for compatibility with leafnode+ */
-	        maxage = atoi( value );
+	        maxage = strtol( value, NULL, 10 );
 		if ( debugmode )
 		    syslog( LOG_DEBUG, "config: maxage is %d", maxage );
 	    }
 	    else if ( strcmp( "maxfetch", param ) == 0 ) {
-	        artlimit = atol( value );
+	        artlimit = strtol( value, NULL, 10 );
 		if ( debugmode )
 		    syslog( LOG_DEBUG, "config: maxfetch is %d", artlimit);
 	    }
 	    else if ( strcmp( "port", param ) == 0 ) {
 		if ( p ) {
-		    p->port = atoi ( value );
+		    p->port = strtol( value, NULL, 10 );
 		    if ( debugmode )
 			syslog( LOG_DEBUG, "config: nntpport is %d",
 			p->port );
@@ -296,13 +299,14 @@ int readconfig( void ) {
 	    }
 	    else if (( strcmp ( "server", param ) == 0 ) ||
 		    ( strcmp ( "supplement", param ) == 0 )) {
-		syslog( LOG_INFO, "config: server is %s", value );
+		if ( debugmode )
+		    syslog( LOG_DEBUG, "config: server is %s", value );
 		p = (struct serverlist *)critmalloc( sizeof(struct serverlist),
 		     "allocating space for server name");
 		p->name = strdup( value );
 		p->descriptions = TRUE;
 		p->next = NULL;
-		p->timeout = 10;	/* default 10 seconds */
+		p->timeout = 30;	/* default 30 seconds */
 		p->port = 0;
 		p->username = NULL;
 		p->password = NULL;
