@@ -64,8 +64,9 @@ int initvars( char * progname );
 /* converts a message-id to a file name, the return value points into
    a static array */
 const char * lookup ( const char *msgid );
-/* reads one line, regardless of length */
-char * getaline(FILE *f);
+
+char * getaline(FILE *f);	/* reads one line, regardless of length */
+char * tgetaline( FILE *f );	/* read line with timer */
 
 /* changes (and optionally creates) directory */
 int chdirgroup(const char *group, int creatdir);
@@ -76,20 +77,20 @@ int chdirgroup(const char *group, int creatdir);
 struct newsgroup {
     struct newsgroup * left;
     struct newsgroup * right;
-    int first;
-    int last;
+    unsigned long first;
+    unsigned long last;
     int alive;
     char * name;
     char * desc;
     time_t age;
 };
 
-int isinteresting( char * groupname );
+int isinteresting( const char * groupname );
 void insertgroup(const char * name, int first, int last, const char * desc,
 	int date );
-struct newsgroup * findgroup(const char* name);
+struct newsgroup * findgroup( const char* name );
 void readactive( void );
-void writeactive( void );
+void writeactive( int forcedwrite );
 void fakeactive( void );
 
 extern struct newsgroup * active;
@@ -109,15 +110,14 @@ struct stringlist {
     struct stringlist * next;
     char string[1] ;
 };
+char * findinlist( struct stringlist * haystack, char * needle ) ;
+	/* find a string in a stringlist by doing a linear search */
+void freelist( struct stringlist * list ) ;
+	/* free memory occupied by a stringlist */
 	
 /*
  * filtering headers for regexp
  */
-struct filterlist {
-    struct filterlist * next;
-    regex_t * expr;
-};
-extern struct filterlist * filter;
 void readfilter( char *filterfile ) ;
 int dofilter( char * h ) ;
 
@@ -156,11 +156,10 @@ struct xoverinfo {
 };
 
 extern struct xoverinfo * xoverinfo;
-extern int xfirst;
-extern int xlast;
+extern long xfirst;
+extern long xlast;
 
 int getxover( void );	/* set xoverinfo, return 0 on error, nonzero else */
-int legalxoverline ( char * xover );	/* return 1 if xoverline is legal */
 
 /*
  * the strings in config.c
@@ -239,14 +238,17 @@ int lockfile_exists( void );
 void putaline( void );
 void retry( void );
 void readexpire( void );
-int lookup_expire(char *group);
 void free_expire( void );
 int readconfig( void );
 void whoami( void );
+void lowercase( char *string );
 char * critmalloc(size_t size, const char* message);
 char * critrealloc(char *a, size_t size, const char* message);
 int ngmatch(const char* pattern, const char* string);
 
+int rename(const char *old, const char *new);
+				/* to avoid barfing of Digital Unix */
+
 /*
  * stuff from nntputil.c
  */
