@@ -37,7 +37,7 @@ See file COPYING for restrictions on the use of this software.
 #include <dirent.h>
 #include <signal.h>
 
-#ifdef HAVE_FNMATCH_H
+#ifdef HAVE_FNMATCH
 #include <fnmatch.h>
 #endif
 
@@ -47,7 +47,7 @@ extern struct state _res;
 
 /* xoverutil global vars */
 struct xoverinfo * xoverinfo;
-int xfirst, xlast;
+long xfirst, xlast;
 
 /* activutil global vars */
 struct newsgroup * active;
@@ -77,6 +77,7 @@ int initvars( char * progname ) {
     mkdir( s, 0775 );
     sprintf( s, "%s/out.going", spooldir );
     mkdir( s, 2755 );
+    chmod( s, S_ISUID | S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH );
 
     if ( progname ) {
 	pw = getpwnam( "news" );
@@ -123,11 +124,11 @@ int lockfile_exists( void ) {
 	    syslog( LOG_ERR, "unable to read lockfile %s, abort ...", lockfile );
 	    return 1;
 	}
-    } else {
-	syslog( LOG_ERR, "unable to open lockfile: %d %m", errno );
+    } else if ( errno != ENOENT ) {
+	syslog( LOG_ERR, "unable to open lockfile %s: %d %m", lockfile, errno );
     }
     if ( ( lf = fopen( lockfile, "w" )) != NULL ) {
-	fprintf( lf, "%d", getpid() );
+	fprintf( lf, "%d", (int)getpid() );
 	fclose( lf );
 	return 0;
     } else {
@@ -142,7 +143,7 @@ int lockfile_exists( void ) {
 /* no good but this server isn't going to be scalable so shut up */
 const char * lookup ( const char *msgid ) {
     static char * name = NULL;
-    static int namelen = 0;
+    static unsigned int namelen = 0; 
     unsigned int r;
     unsigned int i;
 
@@ -263,7 +264,7 @@ int chdirgroup( const char *group, int creatdir ) {
 	    if (*p=='.')
 		*p = '/';
 	    else
-		*p = tolower(*p);
+		*p = tolower((unsigned char)*p);
 	    p++;
 	}
 	if ( !chdir(s) )
@@ -274,7 +275,6 @@ int chdirgroup( const char *group, int creatdir ) {
     return 0;
 }
 
-
 /* get the fully qualified domain name of this box into fqdn */
 
 void whoami( void ) {
@@ -295,6 +295,35 @@ void whoami( void ) {
 	*fqdn = '\0';
 }
 
+/*
+ * find a string in a stringlist
+ * return pointer to string if found, NULL otherwise
+ */
+char * findinlist( struct stringlist * haystack, char * needle ) {
+    struct stringlist * a;
+
+    a = haystack;
+    while ( a && a->string ) {
+	if ( strncmp( needle, a->string, strlen(needle) ) == 0 )
+	    return a->string;
+	a = a->next;
+    }
+    return NULL;
+}
+
+/*
+ * free a list
+ */
+void freelist( struct stringlist * list ) {
+    struct stringlist * a;
+
+    while ( list ) {
+	a = list->next;
+	free( list );
+	list = a;
+    }
+}
+
 /* next few routines implement a mapping from message-id to article
    number, and clearing the entire space */
 
@@ -394,34 +423,30 @@ const char* rfctime(void) {
     time_t now;
     struct tm gmt;
     struct tm local;
-    struct tm conv;
-    signed int diff;
+    int    hours, mins;
+#ifndef HAVE_GMTOFF
+    extern long int timezone;
+#endif
 
     /* get local and Greenwich times */
     now = time(0);
     gmt	  = *(gmtime(&now));
     local = *(localtime(&now));
-    conv  = *(localtime(&now));
 
-    /* if GMT were local, what would it be? */
-    conv.tm_year = gmt.tm_year;
-    conv.tm_mon	 = gmt.tm_mon;
-    conv.tm_mday = gmt.tm_mday;
-    conv.tm_hour = gmt.tm_hour;
-    conv.tm_min	 = gmt.tm_min;
-    conv.tm_sec	 = gmt.tm_sec;
-    conv.tm_wday = gmt.tm_wday;
-    conv.tm_yday = gmt.tm_yday;
-
-    /* see how far the real local is from GMT-as-local */
-    /* should convert to signed int from double properly */
-    diff = (mktime(&local) - mktime(&conv)) / 60.0;
+#ifdef HAVE_GMTOFF
+    hours = local.tm_gmtoff/60/60;
+    mins  = local.tm_gmtoff/60%60;
+#else
+    /* this is empirical */
+    hours = -timezone/60/60+((local.tm_isdst>0)?1:0);
+    mins  = -timezone/60%60;
+#endif
 
     /* finally print the string */
     sprintf(date, "%3s, %d %3s %4d %02d:%02d:%02d %+03d%02d",
 	    days[local.tm_wday], local.tm_mday, months[local.tm_mon],
 	    local.tm_year+1900, local.tm_hour, local.tm_min, local.tm_sec,
-	    diff/60, diff%60);
+	    hours, mins);
 
     return (date);
 }
@@ -482,20 +507,34 @@ char *getaline(FILE *f) {
     return buf;
 }
 
-int ngmatch(const char* pattern, const char* str)
-{
-#ifdef HAVE_FNMATCH_H
-    return fnmatch(pattern, str, 0);
+int ngmatch(const char* pattern, const char* str) {
+#ifdef HAVE_FNMATCH
+    if ( strchr( pattern, '?' ) || !strchr( pattern, '*' ) ||
+	 !strchr( pattern, '[' ) ) {
+	/* pattern contains something wildcard-like */
+	if ( fnmatch( pattern, str, 0 ) == 0 )
+	    return 0;
+	else
+	    return 1;
+    }
+    else {
+	/* no wildcards - we can do case-insensitive comparison */
+	if ( strcasecmp( pattern, str ) == 0 )
+	    return 0;
+	else
+	    return 1;
+    }
 #else
+    /* poor man's fnmatch() - but it is case-insensitive! */
     int len = strlen(pattern);
     if (pattern[len - 1] == '*') {
 	/* If pattern is terminated with '*', treat it as *. */
 	/* Therefore, compare a part of string before '*'. */
-	if (strncmp(pattern, str, len - 1) == 0)
+	if (strncasecmp(pattern, str, len - 1) == 0)
 	    return 0;
     } else {
 	/* Otherwise, treat string as just string. */
-	if (strcmp(pattern, str) == 0)
+	if (strcasecmp(pattern, str) == 0)
 	    return 0;
     }
     return 1;
