@@ -8,6 +8,7 @@
  */
 
 #include "leafnode.h"
+#include <sys/types.h>
 #include <sys/stat.h>
 #include <dirent.h>
 #include <stdio.h>
@@ -22,6 +23,7 @@ int main ( void ) {
     DIR * d;
     struct dirent * de;
     struct stat st;
+    unsigned long filesize;
 
     sprintf( s, "%s/out.going", spooldir );
     if ( chdir(s) < 0 ) {
@@ -33,6 +35,7 @@ int main ( void ) {
 	fprintf( stderr, "Cannot open directory %s -- aborting.\n", s );
 	exit(1);
     }
+
     while ( ( de = readdir(d) ) ) {
 	if ( stat( de->d_name, &st ) ) {
 	    fprintf( stderr, "Cannot stat %s\n", de->d_name );
@@ -40,9 +43,13 @@ int main ( void ) {
 	else if ( S_ISREG( st.st_mode ) ) {
 	    f = fopen( de->d_name, "r" );
 	    if ( f ) {
-		printf("%s in %s\n\t%s\n", fgetheader( f, "From:" ),
+	    	filesize = st.st_size;
+		printf("%s: %8lu bytes, spooled %s\tFrom: %-.66s\n"
+			"\tNgrp: %-.66s\n\tSubj: %-.66s\n",
+			de->d_name, filesize, ctime(&st.st_mtime),
+			fgetheader( f, "From:" ),
 			fgetheader( f, "Newsgroups:" ),
-			fgetheader( f, "Subject:") );
+			fgetheader( f, "Subject:" ) );
 		fclose(f);
 	    }
 	    else
