@@ -34,6 +34,7 @@ See file COPYING for restrictions on the use of this software.
 #include <fcntl.h>
 #include <limits.h>
 #include <netdb.h>
+#include <setjmp.h>
 #include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -47,6 +48,18 @@ See file COPYING for restrictions on the use of this software.
 
 #define MAXLINELENGTH 1000
 
+#ifdef HAVE_IPV6
+/* 
+ *  * local union struct
+ *   */
+union sockaddr_union
+{
+    struct sockaddr         sa;
+    struct sockaddr_in      sin;
+    struct sockaddr_in6     sin6;
+};
+#endif
+
 int write2(int fd, const char *msg);
 int hash (const char *);
 FILE * fopenart(const char *);
@@ -61,7 +74,7 @@ void doarticle(const char *arg, int what);
 void dogroup(const char *);
 void dohelp(void);
 void domove(int);
-void dolist(const char *);
+void dolist(char *p);
 void donewgroups(const char *);
 /* void donewnews(const char*); */
 void dopost( void );
@@ -74,7 +87,7 @@ struct newsgroup * group;	/* current group, initially none */
 struct newsgroup * xovergroup = NULL;
 int pseudogroup;		/* indicates whether we are in a "real"
 				   group or pretend only */
-unsigned long int artno; /* current article number */
+unsigned long artno;		/* current article number */
 char *cmd; /* current command line */
 time_t activetime;
 static int justaftergroup;
@@ -82,6 +95,29 @@ static int justaftergroup;
 int debug = 0;
 int verbose = 0;		/* verbose doesn't count here */
 
+static jmp_buf timeout;
+
+static void timer(int sig) {
+    longjmp(timeout, 1);
+    exit(sig);			/* not reached */
+}
+
+/*
+ * call getaline with 15 min timeout (not configurable)
+ */
+static char * mgetaline( FILE *f ) {
+    char * l;
+
+    if ( setjmp(timeout) ) {
+	 return NULL;
+    }
+    ( void ) signal( SIGALRM, timer );
+    ( void ) alarm( 15*60 );
+    l = getaline( f );
+    ( void ) alarm( 0U );
+    return l;
+}
+
 void rereadactive(void) {
     struct stat st;
 
@@ -108,16 +144,11 @@ void parser(void) {
     char *arg;
     int n;
 
-    while(1) {
-	if (!(cmd=getaline( stdin ))) {
-	    if ( debugmode )
-		syslog( LOG_DEBUG, ">421 Network error" );
-	    error("421 Network error");
-	    return;
-	}
-
+    while ( ( cmd=mgetaline( stdin )) ) {
 	n = strlen(cmd);
-	if ( ( n == 0 ) || ( n > MAXLINELENGTH ) ) {
+	if ( n == 0 )
+	    continue;		/* ignore */
+	if ( n > MAXLINELENGTH ) {
 	    /* ignore attempts at buffer overflow */
 	    if ( debugmode )
 		syslog( LOG_DEBUG, ">500 Dazed and confused" );
@@ -127,9 +158,9 @@ void parser(void) {
 
 	/* parse command line */
 	n = 0;
-	while( isalpha((int)cmd[n]) )
+	while( isalpha((unsigned char)cmd[n]) )
 	    n++;
-	while( isspace((int)cmd[n]) )
+	while( isspace((unsigned char)cmd[n]) )
 	    cmd[n++] = '\0';
 
 	arg = cmd+n;
@@ -137,7 +168,7 @@ void parser(void) {
 	while( cmd[n] )
 	    n++;
 	n--;
-	while(isspace((int)cmd[n]))
+	while(isspace((unsigned char)cmd[n]))
 	    cmd[n--] = '\0';
 
 	if (!strcasecmp(cmd, "quit")) {
@@ -202,6 +233,9 @@ void parser(void) {
 	}
 	fflush(stdout);
     }
+    if ( debugmode )
+	syslog( LOG_DEBUG, ">421 Network error" );
+    error("421 Network error");
 }
 
 /*
@@ -263,7 +297,7 @@ FILE * fopenpseudoart(const char * arg, const unsigned long article_num) {
 	syslog( LOG_DEBUG, "%s: first %lu last %lu artno %lu",
     		group->name, group->first, group->last, article_num );
     if ( article_num && article_num == group->first &&
-         group->first >= group->last ) {
+	  group->first >= group->last ) {
 	f = buildpseudoart(group->name);
     } else if ( !article_num ) {
 	if (!strncmp(arg, "<leafnode:placeholder:", 22)) {
@@ -305,8 +339,6 @@ FILE * fopenart(const char * arg) {
 	f = fopen(lookup(arg), "r");
     } else if (group && artno) {
 	sprintf(s, "%lu", artno);
-	if ( debugmode )
-	    syslog( LOG_DEBUG, "fopenart: %s", s );
 	f = fopen(s, "r");
 	if (!f)
 	    f = fopenpseudoart(s, a);
@@ -330,23 +362,23 @@ static int markdownload( long id )
 
     sprintf( s, "%s/interesting.groups/%s", spooldir, group->name );
     if ( ( f = fopen( s, "r+" ) ) ) {
-        i = 0;
-        while ( fgets( s, 1024, f ) ) {
-            if ( sscanf( s, "%ld", &n ) == 1 && n == id )
-                return 0; /* already marked */
-            ++i;
-        }
-        if ( i < BODY_DOWNLOAD_LIMIT ) {
-            fprintf( f, "%ld\n", id );
+	 i = 0;
+	 while ( fgets( s, 1024, f ) ) {
+	     if ( sscanf( s, "%ld", &n ) == 1 && n == id )
+	         return 0; /* already marked */
+	     ++i;
+	 }
+	 if ( i < BODY_DOWNLOAD_LIMIT ) {
+	     fprintf( f, "%ld\n", id );
 	    if ( debugmode )
-        	syslog( LOG_DEBUG, "Marking %s %ld for download",
-        		group->name, id );
-        }
-        else {
-            syslog( LOG_ERR, "Too many bodies marked in %s",
-                    group->name );
-        }
-        fclose( f );
+	 	syslog( LOG_DEBUG, "Marking %s %ld for download",
+	 		group->name, id );
+	 }
+	 else {
+	     syslog( LOG_ERR, "Too many bodies marked in %s",
+	             group->name );
+	 }
+	 fclose( f );
     }
     return 1;
 }
@@ -410,12 +442,9 @@ void doarticle(const char *arg, int what) {
 		        localartno = strtoul( q, NULL, 10 );
 		}
 	    }
+	    free( p );
 	}
     }
-    if ( p )
-	free( p );
-    if ( q )
-	free( q );
 
     sprintf( s, "%3d %lu %s article retrieved - ", 223-what,
 		localartno, localmsgid );
@@ -479,20 +508,32 @@ void doarticle(const char *arg, int what) {
 
 /* note bug.. need not be _immediately after_ GROUP */
 void markinterest( void ) {
+    struct stat st;
+    struct utimbuf buf;
+    int error;
+    time_t now;
     FILE * f;
 
     if ( !justaftergroup )
 	return;
+    error = 0;
     sprintf( s, "%s/interesting.groups/%s", spooldir, group->name );
-    if ( utime( s, NULL ) ) {
-	if ( errno == ENOENT) {
-	    f = fopen( s, "w" );
-	    if ( f )
-		fclose( f );
-	    else
-		syslog( LOG_ERR, "Could not open %s: %m", s );
-	} else
-	    syslog( LOG_ERR, "Failed to change access time for %s: %m", s );
+    if ( stat( s, &st ) == 0 ) {
+	now = time(0);
+	buf.actime = ( now < st.st_atime ) ? st.st_atime : now;
+			/* now < update may happen through HW failures */
+	buf.modtime = st.st_mtime;
+	if ( utime( s, &buf ) )
+	    error = 1;
+    }
+    else
+	error = 1;
+    if ( error ) {
+	f = fopen( s, "w" );
+	if ( f )
+	    fclose( f );
+	else
+	    syslog( LOG_ERR, "Could not create %s: %m", s );
     }
 
     justaftergroup = FALSE;
@@ -509,10 +550,10 @@ void dogroup(const char *arg) {
 	group = g;
 	if ( chdirgroup( g->name, FALSE ) ) {
 	    if ( debugmode )
-		syslog( LOG_DEBUG, ">211 %d %d %d %s group selected",
+		syslog( LOG_DEBUG, ">211 %lu %lu %lu %s group selected",
 			g->last >= g->first ? g->last-g->first+1 : 0,
 			g->first, g->last, g->name);
-	    printf("211 %d %d %d %s group selected\r\n", 
+	    printf("211 %lu %lu %lu %s group selected\r\n", 
 		   g->last >= g->first ? g->last-g->first+1 : 0, 
 		   g->first, g->last, g->name);
 	    pseudogroup = FALSE;
@@ -538,12 +579,35 @@ void dogroup(const char *arg) {
     }
 }
 
-
 void dohelp(void) {
-    printf("500 Read RFC 977 and 1036 for elucidation\r\n");
+    printf("100 Legal commands\r\n");
+/*  printf("  authinfo user Name|pass Password|generic <prog> <args>\r\n"); */
+    printf("  article [MessageID|Number]\r\n");
+    printf("  body [MessageID|Number]\r\n");
+/*  printf("  date\r\n"); */
+    printf("  group newsgroup\r\n");
+    printf("  head [MessageID|Number]\r\n");
+    printf("  help\r\n");
+    printf("  ihave\r\n");
+    printf("  last\r\n");
+/*  printf("  list [active|newsgroups|distributions|schema] [group_pattern]\r\n"); */
+    printf("  list [active|newsgroups] [group_pattern]\r\n");
+    printf("  listgroup newsgroup\r\n");
+    printf("  mode reader\r\n");
+    printf("  newgroups yymmdd hhmmss [\"GMT\"] [<distributions>]\r\n");
+/*  printf("  newnews newsgroups yymmdd hhmmss [\"GMT\"] [<distributions>]\r\n"); */
+    printf("  next\r\n");
+    printf("  post\r\n");
+    printf("  slave\r\n");
+    printf("  stat [MessageID|Number]\r\n");
+/*  printf("  xgtitle [group_pattern]\r\n"); */
+    printf("  xhdr header [range|MessageID]\r\n");
+    printf("  xover [range]\r\n");
+/*  printf("  xpat header range|MessageID pat [morepat...]\r\n"); */
+/*  printf("  xpath MessageID\r\n"); */
+    printf(".\r\n");
 }
 
-
 void domove( int by ) {
     char * msgid;
 
@@ -601,79 +665,88 @@ void list(struct newsgroup *g, int what, const char * pattern) {
 		printf( "%s\t%s\r\n", g->name, g->desc ? g->desc : "" );
 	} else {
 	    if ( !pattern )
-		printf( "%s %010d %010d y\r\n", g->name, g->last, g->first);
+		printf( "%s %010lu %010lu y\r\n", g->name, g->last, g->first);
 	    else if ( ngmatch( pattern, g->name ) == 0 )
-		printf( "%s %010d %010d y\r\n", g->name, g->last, g->first);
+		printf( "%s %010lu %010lu y\r\n", g->name, g->last, g->first);
 	}
 	list (g->left, what, pattern);
     }
 }
 	
+void dolist( char *arg ) {
+    char * p = NULL;
 
-void dolist(const char *arg) {
     if ( !strcasecmp(arg, "extensions") ) {
 	printf("202 extensions supported follow\r\n"
 	       " OVER\r\n"
 	       " LISTGROUP\r\n"
 	       ".\r\n" );
-        syslog( LOG_DEBUG, ">202 extensions supported follow" );
+	if ( debugmode )
+	     syslog( LOG_DEBUG, ">202 extensions supported follow" );
     } else if ( !strcasecmp(arg, "overview.fmt") ) {
-        printf("215 information follows\r\n"
-               "Subject:\r\n"
-               "From:\r\n"
-               "Date:\r\n"
-               "Message-ID:\r\n"
-               "References:\r\n"
-               "Bytes:\r\n"
-               "Lines:\r\n"
-               "Xref:full\r\n"
-               ".\r\n");
-        syslog( LOG_DEBUG, ">215 information follows" );
+	 printf("215 information follows\r\n"
+	        "Subject:\r\n"
+	        "From:\r\n"
+	        "Date:\r\n"
+	        "Message-ID:\r\n"
+	        "References:\r\n"
+	        "Bytes:\r\n"
+	        "Lines:\r\n"
+	        "Xref:full\r\n"
+	        ".\r\n");
+	if ( debugmode )
+	     syslog( LOG_DEBUG, ">215 information follows" );
     } else if ( !strcasecmp(arg, "active.times") ) {
-        printf("215 Placeholder - Leafnode will fetch groups on demand\r\n"
-               "news.announce.newusers 42 tale@uunet.uu.net\r\n"
-               "news.answers 42 tale@uunet.uu.net\r\n"
-               ".\r\n");
-        syslog( LOG_DEBUG,
-               ">215 Placeholder - Leafnode will fetch groups on demand" );
+	 printf("215 Placeholder - Leafnode will fetch groups on demand\r\n"
+	        "news.announce.newusers 42 tale@uunet.uu.net\r\n"
+	        "news.answers 42 tale@uunet.uu.net\r\n"
+	        ".\r\n");
+	if ( debugmode )
+	    syslog( LOG_DEBUG,
+		    ">215 Placeholder - Leafnode will fetch groups on demand" );
     } else {
-        if (chdir(spooldir)) {
-            printf("503 News spool directory does not exist!\r\n");
-            syslog( LOG_ERR, ">503 News spool directory does not exist!" );
-        } else if (!active) {
-            printf("503 Group information file does not exist!\r\n");
-            syslog( LOG_ERR, ">503 Group information file does not exist!" );
-        } else if ( !*arg || !strncasecmp(arg, "active", 6) ) {
+	if (!active) {
+	     printf("503 Group information file does not exist!\r\n");
+	     syslog( LOG_ERR, ">503 Group information file does not exist!" );
+	 } else if ( !*arg || !strncasecmp(arg, "active", 6) ) {
 	    printf("215 Newsgroups in form \"group high low flags\".\r\n");
-	    syslog( LOG_DEBUG,
-		    ">215 Newsgroups in form \"group high low flags\"." );
-	    if ( !*arg || strlen(arg) == 6 )
-		list( active, 0, NULL );
-	    else {
-		while ( *arg && (!isspace((int)*arg)) )
-		    arg++;
-		while ( *arg && isspace((int)*arg) )
-		    arg++;
-        	list ( active, 0, arg );
+	    if ( debugmode )
+		syslog( LOG_DEBUG,
+			">215 Newsgroups in form \"group high low flags\"." );
+	    if ( active ) {
+		if ( !*arg || strlen(arg) == 6 )
+		    list( active, 0, NULL );
+		else {
+		    while ( *arg && (!isspace((unsigned char)*arg)) )
+			arg++;
+		    while ( *arg && isspace((unsigned char)*arg) )
+			arg++;
+		    p = arg;
+	 	    list ( active, 0, arg );
+		}
 	    }
-            printf(".\r\n");
-        } else if ( !strncasecmp(arg, "newsgroups", 10) ) {
+	     printf(".\r\n");
+	 } else if ( !strncasecmp(arg, "newsgroups", 10) ) {
 	    printf("215 Descriptions in form \"group description\".\r\n");
-	    syslog( LOG_DEBUG,
-		    ">215 Descriptions in form \"group description\"." );
-	    if ( strlen(arg) == 10 )
-		list( active, 1, NULL );
-	    else {
-		while ( *arg && (!isspace((int)*arg)) )
-		    arg++;
-		while ( *arg && isspace((int)*arg) )
-		    arg++;
-        	list ( active, 1, arg );
+	    if ( debugmode )
+		syslog( LOG_DEBUG,
+			">215 Descriptions in form \"group description\"." );
+	    if ( active ) {
+		if ( strlen(arg) == 10 )
+		    list( active, 1, NULL );
+		else {
+		    while ( *arg && (!isspace((unsigned char)*arg)) )
+			arg++;
+		    while ( *arg && isspace((unsigned char)*arg) )
+			arg++;
+	 	    list ( active, 1, arg );
+		}
 	    }
-            printf(".\r\n");
-         } else {
-            printf( "503 Syntax error\r\n" );
-            syslog( LOG_DEBUG, ">503 Syntax error" );
+	     printf(".\r\n");
+	  } else {
+	     printf( "503 Syntax error\r\n" );
+	    if ( debugmode )
+		syslog( LOG_DEBUG, ">503 Syntax error" );
 	 }
     }
 }
@@ -682,7 +755,7 @@ static void helpdonewgroups( struct newsgroup *g, time_t age ) {
     if ( g ) {
 	helpdonewgroups( g->right, age );
 	if ( g->age >= age )
-	    printf( "%s %d %d y\r\n", g->name, g->first, g->last );
+	    printf( "%s %lu %lu y\r\n", g->name, g->first, g->last );
 	helpdonewgroups( g->left, age );
     }
 }
@@ -701,19 +774,28 @@ void donewgroups(const char *arg) {
     year    = ltime->tm_year % 100;
     century = ltime->tm_year / 100;
 
+    memset( &timearray, 0, sizeof(timearray) );
     l = NULL;
     a = strtol( arg, &l, 10 );
     b = a / 10000;
     if ( b < 100 ) {
+	/* YYMMDD */
 	if ( b <= year )
-	    timearray.tm_year = b + century ;
+	    timearray.tm_year = b + (century*100) ;
 	else
-	    timearray.tm_year = b + century - 100 ;
-    } else
+	    timearray.tm_year = b + (century-1)*100 ;
+    } else if ( b < 1000 ) {
+    	/* YYYMMDD - this should not happen, but leafnode will parse it */
+	syslog( LOG_NOTICE,
+		"NEWGROUPS year is %d: please update your newsreader", b );
+	timearray.tm_year = b;
+    } else {
+	/* YYYYMMDD */
 	timearray.tm_year = b - 1900 ;
+    }
     timearray.tm_mon  = (a % 10000 / 100) - 1 ;
     timearray.tm_mday = a % 100 ;
-    while ( *l && isspace((int)*l) )
+    while ( *l && isspace((unsigned char)*l) )
 	l++;
     a = strtol( l, NULL, 10 );	/* we don't care about the rest of the line */
     timearray.tm_hour = a / 10000;
@@ -796,12 +878,11 @@ void dopost( void ) {
 
     out = open( outname, O_WRONLY|O_EXCL|O_CREAT, 0444 );
     if ( out < 0 ) {
-	out = open( outname, O_WRONLY|O_EXCL|O_CREAT, 0444 );
-	if ( out < 0 ) {
-	    printf( "440 Unable to open spool file %s\r\n", outname );
-	    syslog( LOG_ERR, ">440 Unable to open spool file %s", outname );
-	    return;
-	}
+	char *error = strerror(errno);
+	printf( "441 Unable to open spool file %s: %s\r\n", outname, error );
+	syslog( LOG_ERR, ">441 Unable to open spool file %s: %s",
+		outname, error );
+	return;
     }
 
     printf( "340 Go ahead.\r\n" );
@@ -858,7 +939,7 @@ void dopost( void ) {
 	len = strlen( line );
 	/* replace tabs with spaces */
 	for ( i = 0; i < len; i++ )
-	    if ( isspace((int)line[i]) )
+	    if ( isspace((unsigned char)line[i]) )
 		line[i] = ' ';
 	if ( len && line[len-1]=='\n' )
 	    line[--len] = '\0';
@@ -928,8 +1009,8 @@ void dopost( void ) {
     }
 
     unlink( outname );
-    printf( "440 Formatting error, article not posted\r\n" );
-    syslog( LOG_INFO, ">440 Formatting error, article not posted" );
+    printf( "441 Formatting error, article not posted\r\n" );
+    syslog( LOG_INFO, ">441 Formatting error, article not posted" );
 }
 
 
@@ -942,7 +1023,7 @@ void doxhdr(char *arg) {
     int n = 7;
     int dash;
     char * l;
-    int a,b,c;
+    int a,b=0,c;
     FILE * f;
 
     if ( !arg || !*arg ) {
@@ -956,11 +1037,11 @@ void doxhdr(char *arg) {
     }
 
     l = arg;
-    while( l && *l && !isspace((int)*l) )
+    while( l && *l && !isspace((unsigned char)*l) )
 	l++;
     if (l && *l)
 	*l++ = '\0';
-    while( l && *l && isspace((int)*l) )
+    while( l && *l && isspace((unsigned char)*l) )
 	l++;
 
     if (l && *l=='<') { /* handle message-id form (well) */
@@ -978,9 +1059,9 @@ void doxhdr(char *arg) {
 	} while (l && *l && 
 		 strncasecmp(arg, l, strlen(arg)) && 
 		 l[strlen(arg)]!=':'); /* uh, space before : allowed? */
-	while( l && *l && !isspace((int)*l) )
+	while( l && *l && !isspace((unsigned char)*l) )
 	    l++;
-	while( l && *l && isspace((int)*l) )
+	while( l && *l && isspace((unsigned char)*l) )
 	    l++;
 	if ( debugmode ) {
 	    syslog( LOG_DEBUG, ">221 %s header follows:", arg );
@@ -1040,23 +1121,26 @@ void doxhdr(char *arg) {
 	    printf( "1 %d\r\n", 1024 );	/* just a guess */
 	else if ( n == 6 )	/* Lines */
 	    printf( "1 %d\r\n", 22 );		/* from buildpseudoart() */
-        printf( ".\r\n" );
-        return;
+	 printf( ".\r\n" );
+	 return;
     }
 
     dash = 0;
-    if (l && isdigit((int)*l) ) {	/* handle range form */
+    if (l && isdigit((unsigned char)*l) ) {	/* handle range form */
 	a = strtol( l, &l, 10 );
+	if ( a < xfirst )
+	    a = xfirst;
 	if (a && l && *l) {
-	    while (l && isspace((int)*l))
+	    while (l && isspace((unsigned char)*l))
 		l++;
 	    if ( *l == '-' ) {
 	        dash = 1;
+		b = xlast;
 		l++;
 		if ( l && *l )
 		    b = strtol( l, &l, 10 );
 	    }
-	    while (l && isspace((int)*l))
+	    while (l && isspace((unsigned char)*l))
 		l++;
 	    if ( l && *l ) {
 		if ( debugmode )
@@ -1066,7 +1150,7 @@ void doxhdr(char *arg) {
 			"or XHDR header message-id" );
 		return;
 	    }
-	    if ( dash )
+	    if ( dash && b > xlast )
 		b = xlast;
 	}
     } else {
@@ -1157,11 +1241,11 @@ void doxover(const char *arg) {
     l = NULL;
     b = a = strtol( arg, &l, 10 );
     if (a && l && *l) {
-	while (l && isspace((int)*l))
+	while (l && isspace((unsigned char)*l))
 	    l++;
 	if ( *l=='-' )
 	    b = strtol( ++l, &l, 10 );
-	while (l && isspace((int)*l))
+	while (l && isspace((unsigned char)*l))
 	    l++;
 	if ( l && *l ) {
 	    printf( "502 Usage: XOVER first[-last]\r\n" );
@@ -1175,7 +1259,7 @@ void doxover(const char *arg) {
 	pseudogroup = TRUE;
 
     if ( !pseudogroup ) {
-        if ( xovergroup != group && getxover() )
+	 if ( xovergroup != group && getxover() )
 	    xovergroup = group;
 
 	if ( b == 0 )
@@ -1268,7 +1352,7 @@ void dolistgroup( const char * arg ) {
 	if ( debugmode )
 	    syslog( LOG_DEBUG,
 		    ">211 Article list for %s follows (pseudo)", g->name );
-	printf( "%d \r\n", g->last ? g->last : 1 );
+	printf( "%lu \r\n", g->last ? g->last : 1 );
     } else {
 	printf( "211 Article list for %s follows\r\n", g->name );
 	if ( debugmode )
@@ -1286,11 +1370,33 @@ void dolistgroup( const char * arg ) {
 int main( int argc, char ** argv ) {
     int fodder;
     struct hostent *he;
+#ifdef HAVE_IPV6
+    union sockaddr_union su;
+#else
     struct sockaddr_in sa;
+#endif
 
     if ( !initvars( NULL ) )
 	exit( 1 );
 
+#ifdef HAVE_IPV6
+    fodder = sizeof(union sockaddr_union);
+    if (getsockname(0, (struct sockaddr *)&su, &fodder)) {
+	strcpy( fqdn, "localhost" );
+    } else {
+	he = gethostbyaddr((char *)&su.sin6.sin6_addr,
+			   sizeof(su.sin6.sin6_addr),
+			   su.sin.sin_family);
+	if ( he && he->h_name) {
+	    strncpy(fqdn, he->h_name, 63);
+	} else {
+	     if (su.sa.sa_family == AF_INET6)
+	        inet_ntop(AF_INET6, &su.sin6.sin6_addr, fqdn, 63);
+	    else
+		inet_ntop(AF_INET,  &su.sin.sin_addr,   fqdn, 63);
+	}
+    }
+#else
     fodder = sizeof(struct sockaddr_in);
     if (getsockname(0, (struct sockaddr *)&sa, &fodder)) {
 	strcpy( fqdn, "localhost" );
@@ -1302,6 +1408,7 @@ int main( int argc, char ** argv ) {
 		 he && he->h_name ? he->h_name : inet_ntoa(sa.sin_addr),
 		 63 );
     }
+#endif
     if ( strncasecmp( fqdn, "localhost", 9 ) == 0 )
 	whoami();
 
@@ -1315,7 +1422,13 @@ int main( int argc, char ** argv ) {
 	   version, fqdn);
     fflush(stdout);
  
-    readconfig();
+    if ( !readconfig() ) {
+	 printf( "503 Unable to read configuration file, exiting\r\n" );
+	 syslog( LOG_NOTICE, "Reading configuration failed, exiting "
+	 	 "(see syslog for more information).\n");
+	 exit(1);
+    }
+
     pseudogroup = FALSE;
 
     rereadactive();
