@@ -42,7 +42,7 @@ See file COPYING for restrictions on the use of this software.
 
 time_t now;
 
-int default_expire;
+time_t default_expire;
 
 int verbose = 0;
 int debug = 0;
@@ -55,7 +55,6 @@ struct exp {
     int exists;
 };
 
-
 void free_expire( void ) {
     struct expire_entry *a, *b;
 
@@ -71,8 +70,7 @@ void free_expire( void ) {
                         its expire time. Otherwise, return zero.
 */
 
-int lookup_expire(char* group)
-{
+static time_t lookup_expire(char* group) {
     struct expire_entry *a;
    
     a = expire_base;
@@ -84,9 +82,203 @@ int lookup_expire(char* group)
     return 0;
 }
 
+/*
+ * return 1 if xover is a legal overview line, 0 else
+ */
+static int legalxoverline ( char * xover, unsigned long artno ) {
+    char * p;
+    char * q;
 
-static void dogroup(struct newsgroup* g)
-{
+    if ( !xover )
+	return 0;
+
+    /* anything that isn't tab, printable ascii, or latin-* ? then killit */
+
+    p = xover;
+    while ( *p ) {
+	int c = (unsigned char)*p++;
+
+	if ( ( c != '\t' && c < ' ' ) || ( c > 126 && c < 160 ) ) {
+	    if ( debugmode )
+		syslog( LOG_DEBUG,
+			"%lu xover error: non-printable chars.", artno );
+	    return 0;
+	}
+    }
+
+    p = xover;
+    q = strchr( p, '\t' );
+    if ( !q ) {
+	if ( debugmode )
+	    syslog( LOG_DEBUG, "%lu xover error: no Subject: header.", artno );
+	return 0;
+    }
+
+    /* article number */
+
+    while( p != q ) {
+	if ( !isdigit((unsigned char)*p) ) {
+	    if ( debugmode )
+        	syslog( LOG_DEBUG, "%lu xover error: article "
+			"number must consists of digits.", artno );
+	    return 0;
+	}
+	p++;
+    }
+
+    p = q+1;
+    q = strchr( p, '\t' );
+    if ( !q ) {
+	if ( debugmode )
+	    syslog( LOG_DEBUG, "%lu xover error: no From: header.", artno );
+	return 0;
+    }
+
+    /* subject: no limitations */
+
+    p = q+1;
+    q = strchr( p, '\t' );
+    if ( !q ) {
+	if ( debugmode )
+            syslog( LOG_DEBUG, "%lu xover error: no Date: header.", artno );
+	return 0;
+    }
+
+    /* from: no limitations */
+
+    p = q+1;
+    q = strchr( p, '\t' );
+    if ( !q ) {
+	if ( debugmode )
+            syslog( LOG_DEBUG,
+		    "%lu xover error: no Message-ID: header.", artno );
+	return 0;
+    }
+
+    /* date: no limitations */
+
+    p = q+1;
+    q = strchr( p, '\t' );
+    if ( !q ) {
+	if ( debugmode )
+	    syslog( LOG_DEBUG,
+		    "%lu xover error: no References: or Bytes: header.",
+		    artno );
+	return 0;
+    }
+
+    /* message-id: <*@*> */
+
+    if ( *p != '<' ) {
+	if ( debugmode )
+	    syslog( LOG_DEBUG,
+		    "%lu xover error: Message-ID does not start with <.",
+		    artno );
+	return 0;
+    }
+    while ( p != q && *p != '@' && *p != '>' && *p != ' ' )
+	p++;
+    if ( *p != '@' ) {
+	if ( debugmode )
+	    syslog( LOG_DEBUG,
+		    "%lu xover error: Message-ID does not contain @.", artno );
+	return 0;
+    }
+    while ( p != q && *p != '>' && *p != ' ' )
+	p++;
+    if ( *p != '>' ) {
+	if ( debugmode )
+	    syslog( LOG_DEBUG,
+		    "%lu xover error: Message-ID does not end with >.", artno );
+	return 0;
+    }
+    if ( ++p != q ) {
+	if ( debugmode )
+	    syslog( LOG_DEBUG,
+		    "%lu xover error: Message-ID does not end with >.", artno );
+	return 0;
+    }
+
+    p = q+1;
+    q = strchr( p, '\t' );
+    if ( !q ) {
+	if ( debugmode )
+	    syslog( LOG_DEBUG, "%lu xover error: no Bytes: header.", artno );
+	return 0;
+    }
+
+    /* references: a series of <*@*> separated by space */
+
+    while ( p != q ) {
+	if ( *p != '<' ) {
+	    if ( debugmode )
+        	syslog( LOG_DEBUG, "%lu xover error: "
+			"Reference does not start with <.", artno );
+	    return 0;
+	}
+	while ( p != q && *p != '@' && *p != '>' && *p != ' ' )
+	    p++;
+	if ( *p != '@' ) {
+	    if ( debugmode )
+        	syslog( LOG_DEBUG,
+			"%lu xover error: Reference does not contain @.",
+			artno );
+	    return 0;
+	}
+	while ( p != q && *p != '>' && *p != ' ' )
+	    p++;
+	if ( *p++ != '>' ) {
+	    if ( debugmode )
+        	syslog( LOG_DEBUG,
+			"%lu xover error: Reference does not end with >.",
+			artno );
+	    return 0;
+	}
+	while ( p != q && *p == ' ' )
+	    p++;
+    }
+
+    p = q+1;
+    q = strchr( p, '\t' );
+    if ( !q ) {
+	if ( debugmode )
+	    syslog( LOG_DEBUG, "%lu xover error: no Lines: header.", artno );
+	return 0;
+    }
+
+    /* byte count */
+
+    while( p != q ) {
+	if ( !isdigit((unsigned char)*p) ) {
+	    if ( debugmode )
+		syslog( LOG_DEBUG, "%lu xover error: illegal digit "
+			"in Bytes: header.", artno );
+	    return 0;
+	}
+	p++;
+    }
+
+    p = q+1;
+    q = strchr( p, '\t' );
+    if ( q )
+        *q = '\0'; /* kill any extra fields */
+
+ /* line count */
+
+    while( p && *p && p != q ) {
+	if ( !isdigit((unsigned char)*p) ) {
+	    if ( debugmode )
+        	syslog( LOG_DEBUG, "%lu xover error: illegal digit "
+			"in Lines: header.", artno );
+	    return 0;
+	}
+	p++;
+    }
+
+    return 1;
+}
+
+static void dogroup(struct newsgroup* g) {
     char gdir[PATH_MAX];
     char *p;
     char *q;
@@ -119,7 +311,10 @@ static void dogroup(struct newsgroup* g)
 
     first = ULONG_MAX;
     last = 0;
-    while ((de = readdir(d)) != 0) {
+    while ( ( de = readdir(d) ) != 0 ) {
+	if ( !isdigit((unsigned char)de->d_name[0]) ||
+	     stat( de->d_name, &st ) || !S_ISREG( st.st_mode ) )
+	    continue;
 	art = strtoul(de->d_name, &p, 10);
 	if (p && !*p) {
 	    if (art < first)
@@ -136,8 +331,10 @@ static void dogroup(struct newsgroup* g)
     if ( verbose > 1 )
         printf("%s: low water mark %lu, high water mark %lu\n",
 	        g->name, first, last );
-    syslog( LOG_INFO, "%s: expire %lu, low water mark %lu, high water mark %lu",
-            g->name, expire, first, last );
+    if ( debugmode )
+	syslog( LOG_DEBUG,
+		"%s: expire %lu, low water mark %lu, high water mark %lu",
+		g->name, expire, first, last );
 
     if ( expire <= 0 )
 	return;
@@ -174,7 +371,7 @@ static void dogroup(struct newsgroup* g)
 
 	p = overview;
 	while (p && *p) {
-	    while (p && isspace((int)*p))
+	    while (p && isspace((unsigned char)*p))
 		p++;
 	    art = strtoul(p, NULL, 10);
 	    if (art >= first && art <= last &&
@@ -196,8 +393,7 @@ static void dogroup(struct newsgroup* g)
 
     for (art = first; art <= last; art++) {
 	if (articles[art - first].xover &&
-	    !legalxoverline(articles[art - first].xover)) {
-	    syslog(LOG_INFO, "bad overview line for %s/%lu", g->name, art);
+	    !legalxoverline(articles[art - first].xover, art)) {
 	    articles[art - first].xover = NULL;
 	}
     }
@@ -270,13 +466,13 @@ static void dogroup(struct newsgroup* g)
 		sprintf(name, "%lu", art);
 		if (!unlink(name)) {
 		    if ( debugmode )
-			syslog( LOG_DEBUG, "deleted article %s/%d", gdir, art );
+			syslog( LOG_DEBUG, "deleted article %s/%ld", gdir, art );
 		    deleted++;
 		} else if (errno != ENOENT && errno != EEXIST) {
 		    /* if file was deleted alredy or it was not a file */
 		    /* but a directory, skip error message */
 		    kept++;
-		    syslog( LOG_ERR, "unlink %s/%d: %m", gdir, art );
+		    syslog( LOG_ERR, "unlink %s/%ld: %m", gdir, art );
 	        } else {
 		    /* deleted by someone else */
 		}
@@ -298,13 +494,15 @@ static void dogroup(struct newsgroup* g)
     }
 
     if ( !kept ) {
-        /* reset article counters */
-        g->first = 1;
-	g->last = 1;
 	if ( unlink( ".overview" ) < 0 )
 	    syslog( LOG_ERR, "unlink %s/.overview: %m", gdir );
-	if ( !chdir("..") )
-	    (void) rmdir( gdir );
+	if ( !chdir("..") && ( isinteresting(g->name) == 0 ) ) {
+	    /* delete directory and empty parent directories */
+	    while ( rmdir( gdir ) == 0 ) {
+		getcwd( gdir, PATH_MAX );
+		chdir("..");
+	    }
+	}
     }
 }
     
@@ -418,7 +616,7 @@ int main(int argc, char** argv)
     default_expire = expire;
 
     expiregroup( active );
-    writeactive();
+    writeactive( 0 );
     unlink( lockfile );
     expiremsgid();
     free_expire();
