@@ -45,167 +45,6 @@ See file COPYING for restrictions on the use of this software.
 
 extern struct state _res;
 
-/* return 1 if xover is a legal overview line, 0 else */
-int legalxoverline ( char * xover ) {
-    char * p;
-    char * q;
-
-    if ( !xover )
-	return 0;
-
-    /* anything that isn't tab, printable ascii, or latin-* ? then killit */
-
-    p = xover;
-    while ( *p ) {
-	int c = (unsigned char)*p++;
-
-	if ( ( c != '\t' && c < ' ' ) || ( c > 126 && c < 160 ) ) {
-	    syslog( LOG_INFO, "xover error: non-printable chars.");
-	    return 0;
-	}
-    }
-
-    p = xover;
-    q = strchr( p, '\t' );
-    if ( !q ) {
-	syslog( LOG_INFO, "xover error: no Subject: header.");
-	return 0;
-    }
-
-    /* article number */
-
-    while( p != q ) {
-	if ( !isdigit((int)*p) ) {
-            syslog( LOG_INFO, "xover error: "
-                              "article number must consists of digits.");
-	    return 0;
-	}
-	p++;
-    }
-
-    p = q+1;
-    q = strchr( p, '\t' );
-    if ( !q ) {
-	syslog( LOG_INFO, "xover error: no From: header.");
-	return 0;
-    }
-
-    /* subject: no limitations */
-
-    p = q+1;
-    q = strchr( p, '\t' );
-    if ( !q ) {
-        syslog( LOG_INFO, "xover error: no Date: header.");
-	return 0;
-    }
-
-    /* from: no limitations */
-
-    p = q+1;
-    q = strchr( p, '\t' );
-    if ( !q ) {
-        syslog( LOG_INFO, "xover error: no Message-ID: header.");
-	return 0;
-    }
-
-    /* date: no limitations */
-
-    p = q+1;
-    q = strchr( p, '\t' );
-    if ( !q ) {
-	syslog( LOG_INFO, "xover error: no References: or Bytes: header.");
-	return 0;
-    }
-
-    /* message-id: <*@*> */
-
-    if ( *p != '<' ) {
-        syslog( LOG_INFO, "xover error: Message-ID does not start with <.");
-	return 0;
-    }
-    while ( p != q && *p != '@' && *p != '>' && *p != ' ' )
-	p++;
-    if ( *p != '@' ) {
-        syslog( LOG_INFO, "xover error: Message-ID does not contain @.");
-	return 0;
-    }
-    while ( p != q && *p != '>' && *p != ' ' )
-	p++;
-    if ( *p != '>' ) {
-        syslog( LOG_INFO, "xover error: Message-ID does not end with >.");
-	return 0;
-    }
-    if ( ++p != q ) {
-        syslog( LOG_INFO, "xover error: Message-ID does not end with >.");
-	return 0;
-    }
-
-    p = q+1;
-    q = strchr( p, '\t' );
-    if ( !q ) {
-	syslog( LOG_INFO, "xover error: no Bytes: header.");
-	return 0;
-    }
-
-    /* references: a series of <*@*> separated by space */
-
-    while ( p != q ) {
-	if ( *p != '<' ) {
-            syslog( LOG_INFO, "xover error: Reference does not start with <.");
-	    return 0;
-	}
-	while ( p != q && *p != '@' && *p != '>' && *p != ' ' )
-	    p++;
-	if ( *p != '@' ) {
-            syslog( LOG_INFO, "xover error: Reference does not contain @.");
-	    return 0;
-	}
-	while ( p != q && *p != '>' && *p != ' ' )
-	    p++;
-	if ( *p++ != '>' ) {
-            syslog( LOG_INFO, "xover error: Reference does not end with >.");
-	    return 0;
-	}
-	while ( p != q && *p == ' ' )
-	    p++;
-    }
-
-    p = q+1;
-    q = strchr( p, '\t' );
-    if ( !q ) {
-        syslog( LOG_INFO, "xover error: no Lines: header.");
-	return 0;
-    }
-
-    /* byte count */
-
-    while( p != q ) {
-	if ( !isdigit((int)*p) ) {
-	    syslog( LOG_INFO, "xover error: illegal digit in Bytes: header.");
-	    return 0;
-	}
-	p++;
-    }
-
-    p = q+1;
-    q = strchr( p, '\t' );
-    if ( q )
-        *q = '\0'; /* kill any extra fields */
-
- /* line count */
-
-    while( p && *p && p != q ) {
-	if ( !isdigit((int)*p) ) {
-            syslog( LOG_INFO, "xover error: illegal digit in Lines: header.");
-	    return 0;
-	}
-	p++;
-    }
-
-    return 1;
-}
-
-
 
 void stripspace( char * p );
 void stripspace( char * p ) {
@@ -214,11 +53,11 @@ void stripspace( char * p ) {
 
     p1 = p2 = p;
     while ( *p1 ) {
-	if ( isspace((int)*p1) ) {
+	if ( isspace((unsigned char)*p1) ) {
 	    *p2 = ' ';
 	    do {
 		p1++;
-	    } while ( isspace((int)*p1) );
+	    } while ( isspace((unsigned char)*p1) );
 	    if ( *p1 )
 		p2++;
 	} else {
@@ -266,7 +105,7 @@ char * getxoverline( const char * filename ) {
 	    } else if ( !body && !*l ) {
 		linecount = 0;
 		body = 1;
-	    } else if ( h && isspace((int)*l) ) {
+	    } else if ( h && isspace((unsigned char)*l) ) {
 		stripspace( l );
 		if ( *l ) {
 		    (*h) = critrealloc( *h, strlen( *h ) + strlen( l ) + 1,
@@ -313,7 +152,7 @@ char * getxoverline( const char * filename ) {
 		l += 7;
 		stripspace( l );
 		if ( *l ) {
-		    lines = atoi( l );
+		    lines = strtol( l, NULL, 10 );
 /* not strictly correct, but there are probably no multiline Line: headers
 		    h = &lines;
 */
@@ -365,7 +204,7 @@ int getxover( void ) {
     struct dirent * de;
     int fd;
     struct stat st;
-    int art;
+    long art;
     static char * overview;
     int error;
     char * p, * q;
@@ -436,7 +275,7 @@ int getxover( void ) {
 
 	p = overview;
 	while ( p && *p ) {
-	    while ( p && isspace((int)*p) )
+	    while ( p && isspace((unsigned char)*p) )
 		p++;
 	    q = strchr( p, '\n' );
 	    if ( q )
@@ -459,15 +298,13 @@ int getxover( void ) {
     /* so, what was missing? */
 
     while ( (de=readdir(d)) ) {
+      if (!strcmp(".", de->d_name) || !strcmp("..", de->d_name)) continue;
 	art = strtol( de->d_name, &p, 10 );
 	if (p && !*p && art>=xfirst && art<=xlast) {
 	    if ( !xoverinfo[art-xfirst].text ) {
 		if ( debugmode )
 		    syslog( LOG_DEBUG, "reading XOVER info from %s/%s",
 			    getcwd( s, 1024 ), de->d_name );
-		if ( verbose > 2 )
-		    printf( "reading XOVER info from %s/%s\n",
-			    getcwd( s, 1024 ), de->d_name );
 		error++;
 		if ( ( xoverinfo[art-xfirst].text =
 		      getxoverline( de->d_name )) != NULL ) {
@@ -478,31 +315,39 @@ int getxover( void ) {
 		    getcwd( s, 1024 );
 		    syslog( LOG_NOTICE, "illegal article: %s/%s: %m",
 			    s, de->d_name );
-/* This code apparently leads sometimes to the loss of whole directories --
-   removed!
-		    (void) unlink( de->d_name );
-*/
+		    if ( lstat( de->d_name, &st ) && S_ISREG( st.st_mode ) ) {
+			if ( unlink( de->d_name ) )
+			    syslog( LOG_WARNING, "failed to remove %s/%s: %m",
+				    s, de->d_name );
+		    } else {
+			syslog( LOG_WARNING, "%s/%s is not a regular file",
+				s, de->d_name );
+		    }
 		}
 	    }
 	}
 	if ( art >= xfirst && art <= xlast && xoverinfo[art-xfirst].text ) {
 	    xoverinfo[art-xfirst].exists = 1;
 	} else if ( verbose ) {
+	    getcwd( s, 1024 );
 	    if ( art == 0 && strcmp( de->d_name, ".overview" ) &&
 		 strcmp( de->d_name, "." ) && strcmp( de->d_name, ".." ) ) {
 		if ( verbose && unlink( de->d_name ) &&
 		     errno != EISDIR && errno != EPERM )
-		    printf( "attempted to delete %s in case it's junk: %s\n",
-			    de->d_name, errno ? strerror(errno) : "OK" );
-	    } else if ( art > 0 && art < xfirst )
-		printf( "article %d is below the low-water mark (%d)\n",
-			art, xfirst );
-	    else if ( art > xlast )
-		printf( "article %d is above the high-water mark (%d)\n",
-			art, xlast );
-	    else if ( p && !*p )
-		printf( "article %d contained illegal headers\n",
-			art );
+		    printf( "attempted to delete %s/%s in case it's junk: %s\n",
+			    s, de->d_name, errno ? strerror(errno) : "OK" );
+	    } else if ( p && !*p ) {
+	    	if ( art > 0 && art < xfirst )
+		    printf("article %s/%ld is below the low-water mark (%ld)\n",
+			   s, art, xfirst );
+		else if ( art > xlast )
+		    printf("article %s/%ld is above the high-water mark (%ld)\n",
+			   s, art, xlast );
+		else {
+		    printf( "article %s/%ld contained illegal headers\n",
+			    s, art );
+		}
+	    }
 	}
     }
 
@@ -514,11 +359,15 @@ int getxover( void ) {
 	char newfile[20];
 
 	if ( verbose )
-	    printf( "corrected %d lines in %s/.overview\n",
+	    syslog( LOG_DEBUG, "updated %d lines in %s/.overview\n",
 		    error, getcwd( s, 1024 ) );
 
 	strcpy( newfile, ".overview.XXXXXX" );
+#ifdef HAVE_MKSTEMP
+	if ( (wfd=mkstemp(newfile)) != -1) {
+#else
 	if ( (wfd=open( mktemp(newfile), O_WRONLY|O_CREAT|O_EXCL, 0664 )) ) {
+#endif
 	    struct iovec oooh[UIO_MAXIOV];
 	    int vi, vc, va;
 
@@ -561,8 +410,8 @@ int getxover( void ) {
 				"rename(%s/%s, .overview) failed: %m",
 				getcwd(s, 1024), newfile );
 		} else {
-		    if ( verbose )
-			printf( "wrote %s/.overview\n",
+		    if ( debugmode )
+			syslog( LOG_DEBUG, "wrote %s/.overview\n",
 				getcwd( s, 1024 ) );
 		}
 	    } else {
@@ -571,8 +420,12 @@ int getxover( void ) {
 		   .overview file I think */
 	    }
 	} else {
+#ifdef HAVE_MKSTEMP
+	    syslog( LOG_ERR, "mkstemp of new .overview failed: %m" );
+#else
 	    syslog( LOG_ERR, "open(O_WRONLY|O_CREAT|O_EXCL) of new "
 		    ".overview failed: %m" );
+#endif
 	}
     }
     return 1;
